package it.amattioli.workstate.specifications;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.specifications.ChainedSpecification;
import it.amattioli.dominate.specifications.SpecificationsConfig;
import it.amattioli.workstate.config.Registry;
import it.amattioli.workstate.core.Machine;
import it.amattioli.workstate.core.MetaState;

public abstract class MachineSpecification <T extends Entity<?>> extends ChainedSpecification<T> {
	private String propertyName;
	private String workflowName;
	private String stateName;
	
	public static <T extends Entity<?>> MachineSpecification<T> newInstance(String propertyName, String workflowName) {
		MachineSpecification<T> result = null;
		for (Class<? extends MachineSpecification> curr: SpecificationsConfig.getSpecificationClasses(MachineSpecification.class)) {
			Constructor<? extends MachineSpecification> c;
			try {
				c = curr.getConstructor(String.class, String.class, MachineSpecification.class);
			} catch (SecurityException e) {
				throw new RuntimeException(e);
			} catch (NoSuchMethodException e) {
				throw new RuntimeException(e);
			}
			try {
				result = (MachineSpecification<T>)c.newInstance(propertyName, workflowName, result);
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (InstantiationException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			} catch (InvocationTargetException e) {
				throw new RuntimeException(e);
			}
		}
		return result;
	}
	
	public MachineSpecification(String propertyName, String workflowName) {
		this.propertyName = propertyName;
		this.workflowName = workflowName;
	}
	
	public MachineSpecification(String propertyName, String workflowName, MachineSpecification<T> chained) {
		super(chained);
		this.propertyName = propertyName;
		this.workflowName = workflowName;
	}

	protected String getPropertyName() {
		return propertyName;
	}

	public String getStateName() {
		return stateName;
	}

	public void setStateName(String stateName) {
		this.stateName = stateName;
		if (getNextInChain() != null) {
			((MachineSpecification<T>)getNextInChain()).setStateName(stateName);
		}
	}
	
	protected MetaState getMetaState() {
		return Registry.instance().getMetaMachine(workflowName).findMetaState(stateName);
	}

	@Override
	public boolean isSatisfiedBy(T entity) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			Machine m = (Machine)PropertyUtils.getProperty(entity, getPropertyName());
			return m.isInState(getMetaState());
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	public boolean wasSet() {
		return getStateName() != null && !getStateName().equals("");
	}
}
